function [ranks, dist] = crowding_distance_sort(pop, data, front)
%
%
F = data.F;
M = pop.prob.nf;
L = length(F(front).f);					% no of elements in the front
if L == 1
	ranks = F(front).f;
    dist = Inf;
else
	dist = zeros(L, 1);				% vector to store crowding distance
        
    % handle duplicate points correctly
    f = pop.f(F(front).f(:),:);
    [tmp, ID] = sort(f(:,1));
   
    duplicates = [];
    for i = 1:L
        duplicates(i).id = [];
    end
    
    % find duplicates
    i = 1;
    j = 2;
    while i <= L && j <= L
        if norm(f(ID(j),:)-f(ID(i),:), 2) == 0
            duplicates(ID(i)).id = [duplicates(ID(i)).id ID(j)];
            j = j+1;
        else
            i = j;
            j = i+1;
        end
    end
   
    % create from --> to mapping for duplicates
    unique_ids = F(front).f;
    replace = [];
    count = 0;
    for i = 1:L
        dups = duplicates(i).id;
        if ~isempty(dups)
            count = count+1;
            replace(count).from = F(front).f(i);
            replace(count).to = [];
            for j = 1:length(dups)
                id = duplicates(i).id(j);
                replace(count).to = [replace(count).to F(front).f(id)];
                index = find(unique_ids == F(front).f(id));
                unique_ids(index) = [];
            end
            %fprintf('duplidate: %d --> %d\n', replace(count).from, replace(count).to);
        end
    end

    % Use only unique points and insert duplicates
	for i = 1:M
        f = pop.f(unique_ids(:), i);			% get ith objective
		[tmp, I] = sort(f);

        % insert duplicates after the original
        II = unique_ids(I(:)');
        for j = 1:count
            index = find(II == replace(j).from);
            II = [II(1:index) replace(j).to II(index+1:end)];
        end
        [tmp, I] = sort(II);
        I = zeros(1,L);
        for j = 1:L
            I(j) = find(F(front).f == II(j));
        end

        % calculate crowding distance
        f = pop.f(F(front).f(:),i);
		scale = f(I(L)) - f(I(1));
		if scale == 0, scale = 1; end
		dist(I(1)) = Inf;
		for j = 2:L-1
			id = I(j);
			id1 = I(j-1);
			id2 = I(j+1);
%             fprintf('%d: %d: %g %g --> ', i-1, j-1, f(id), dist(id));
			dist(id) = dist(id) + abs(f(id2)-f(id1)) / scale;
%             fprintf('%g\n', dist(id));
		end
	end
	dist = dist / M;
    [tmp, I] = sort(dist, 'descend');
	ranks = F(front).f(I(:));
end
return
