function [childpop] = evolve_localsurrgrad(pop, user_data)

arc = user_data;
eps = 1.e-6;

childpop = evolve_realga(pop, []);

N = pop.size;

count = 0;
for i = 1:N
	x1 = pop.x(i,:);
	x1_normal = normalize(arc.prob, x1);

	% build a local surrogate
	[surr, range] = local_surrogate(arc, x1_normal);

	% do a gradient search
	[x2_normal,f2,eflag,output] = fmincon(@(x) obj_func(x, pop, surr), x1_normal, ...
			[], [], [], [], range(1,:), range(2,:), ...
			@(x) constr_func(x, pop, surr), ...
			optimset('Display', 'off', 'LargeScale', 'off', 'MaxIter', 20));

	x2 = denormalize(arc.prob, x2_normal);
	x2_valid = 0;

	% check the result of gradient solver
	x2_valid = 1;
	if eflag >= 0
		found = 0;
		for j = 1:count
			% check if we already have a solution close to this one
			if norm(x2 - childpop.x(N-j+1,:), 2) < eps
				found = 1;
				break
			end
		end
		if found == 1
			x2_valid = 0;
		end
	end

	% check if the solution is within bounds
	if in_range(arc.prob, x2) == 0
		x2_valid = 0;
	end

	if x2_valid == 1
		fprintf('\t\t%g --> %g (%d) Fn: %d\n', pop.f(i,:), f2, eflag, output.funcCount);
		childpop.x(N-count,:) = x2;
		count = count+1;
	end
end
fprintf('\tMinimized Count = %d\n', count);
return



%% build local surrogate and calculate the range of influence
function [surr, range] = local_surrogate(arc, x_normal)

if arc.traincount > arc.algo.max_traincount
	N = arc.algo.max_traincount;
else
	N = arc.traincount;
end

delta_x = arc.x - repmat(x_normal, arc.count, 1);
dist = sum(delta_x .* delta_x, 2);
[tmp, I] = sort(dist);

surr = surrogate(arc.algo);
[surr, mse] = train(surr, arc.x(I(1:N),:), arc.f(I(1:N),:), arc.g(I(1:N),:), N);
range = minmax(arc.x(I(1:N),:)')';

% Check individual ranges
delta = range(2,:) - range(1,:);
for i = 1:length(delta)
	if delta(i) < 0.001
		range(1,i) = x_normal(i) - 0.001;
		if range(1,i) < 0, range(1,i) = 0; end
		range(2,i) = x_normal(i) + 0.001;
		if range(2,i) > 1, range(2,i) = 1; end
	end
end
%disp(range);
%range(1,:) = denormalize(arc.prob, range(1,:));
%range(2,:) = denormalize(arc.prob, range(2,:));
return


function [obj] = obj_func(x_normal, pop, surr)
%disp(x_normal);
%x_normal = normalize(pop.prob, x);
[f, g, valid] = predict(surr, x_normal);
obj = f;
return


function [c, ceq] = constr_func(x_normal, pop, surr);
%x_normal = normalize(pop.prob, x);
[f, g, valid] = predict(surr, x_normal);
c = -g;
ceq = [];
return
