function childpop = evolve_realga_constraint(pop, user_data)
% EVOLVE_BASICGA(pop)
%		childpop = evolve_nsga2(pop) uses SBX and mutation operators to create
% child population

if sum(pop.feas) ~= round((1-pop.algo.infeasible_ratio)*pop.size)
	childpop = evolve_realga(pop, user_data);
	return
end

childpop = population(pop);

tmp = randperm(pop.size);
a1 = [tmp tmp tmp tmp tmp tmp tmp tmp tmp tmp tmp];

range = pop.prob.range;
nx = pop.prob.nx;

nc = 0;
i = 1;
while nc < pop.size
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a1(i+3));
	if pop.feas(parent1) + pop.feas(parent2) ~= 0
		[c1, c2] = recombination(pop, parent1, parent2, range, nx);
		childpop.x(nc+1,:) = c1;
		childpop.x(nc+2,:) = c2;
		childpop.tracking(nc+1) = 1 + pop.feas(parent1) + pop.feas(parent2);
		childpop.tracking(nc+2) = childpop.tracking(nc+1);
		nc = nc + 2;
	end
	i = i + 4;
	if i > length(a1)-4
		disp([nc i]);
		error('Ran out of solutions');
	end
end

childpop = mutation(childpop, range, nx);


%
% Binary Tournament 
%
function parent = tournament(pop, id1, id2)
if pop.rank(id1) < pop.rank(id2)
	parent = id1;
elseif pop.rank(id2) < pop.rank(id1)
	parent = id2;
elseif rand(1) <= 0.5
	parent = id1;
else
	parent = id2;
end
return


%
% Recombination 
%
function [c1, c2] = recombination(pop, id1, id2, range, nx)
c1 = zeros(nx, 1);
c2 = zeros(nx, 1);

prob_re = pop.algo.prob_re;
eta_re = pop.algo.eta_re;
eta_mc = pop.algo.eta_mc;

do_mc = 0;
if pop.feas(id1) + pop.feas(id2) == 0 && rand(1) < 0
    do_mc = 1;
end

if rand(1) <= prob_re
    for i = 1:nx
		if rand(1) <= 0.5
            if do_mc == 1
                [y1, y2] = op_SBX(pop.x(id1,i), pop.x(id2,i), ...
                        range(i,1), range(i,2), eta_mc);
            else 
                [y1, y2] = op_SBX(pop.x(id1,i), pop.x(id2,i), ...
                        range(i,1), range(i,2), eta_re);
            end
		else
			y1 = pop.x(id1,i);
			y2 = pop.x(id2,i);
		end
		c1(i) = y1;
		c2(i) = y2;
    end
else
    c1 = pop.x(id1,:);
    c2 = pop.x(id2,:);
end

return


%
% Mutation
%
function pop = mutation(pop, range, nx)
prob_mu = pop.algo.prob_mu;
eta_mu = pop.algo.eta_mu;

for i = 1:pop.size
    for j = 1:nx
        x_min = range(j,1);
    	x_max = range(j,2);	
		if rand(1) <= prob_mu
			pop.x(i,j) = op_MUTATE(pop.x(i,j), x_min, x_max, eta_mu);
		end
	end
end
return
