function childpop = evolve_realga(pop, user_data)
% EVOLVE_BASICGA(pop)
%		childpop = evolve_nsga2(pop) uses SBX and mutation operators to create
% child population
childpop = population(pop);

count = floor(pop.size/4 + 0.5);
count2 = floor(pop.size/4);
a1 = randperm(pop.size);
a2 = randperm(pop.size);

range = pop.prob.range;
nx = pop.prob.nx;

% Handle modulo 4 solutions
for i = 1:4:count2*4
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a1(i+3));
	[c1, c2] = recombination(pop, parent1, parent2, range, nx);
	childpop.x(i,:) = c1; 
	childpop.x(i+1,:) = c2;
	childpop.tracking(i) = 1 + pop.feas(parent1) + pop.feas(parent2);
	childpop.tracking(i+1) = childpop.tracking(i);
	parent1 = tournament(pop, a2(i)  , a2(i+1));
	parent2 = tournament(pop, a2(i+2), a2(i+3));
	[c1, c2] = recombination(pop, parent1, parent2, range,  nx);
	childpop.x(i+2,:) = c1;
	childpop.x(i+3,:) = c2;
	childpop.tracking(i+2) = 1 + pop.feas(parent1) + pop.feas(parent2);
	childpop.tracking(i+3) = childpop.tracking(i+2);
end

% Handle extra solutions 
if count*4 > pop.size
	tmp = randperm(pop.size);
	a1 = [a1 tmp(1:count*4-pop.size)];
	tmp = randperm(pop.size);
	a2 = [a2 tmp(1:count*4-pop.size)];

	i = count2*4+1;
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a1(i+3));
	[c1, c2] = recombination(pop, parent1, parent2, range, nx);
	childpop.x(i,:) = c1;
	childpop.tracking(i) = 1 + pop.feas(parent1) + pop.feas(parent2);
	if i+1 <= pop.size
		childpop.x(i+1,:) = c2; 
		childpop.tracking(i+1) = childpop.tracking(i);
	end
	if i+2 <= pop.size
		parent1 = tournament(pop, a2(i)  , a2(i+1));
		parent2 = tournament(pop, a2(i+2), a2(i+3));
		[c1, c2] = recombination(pop, parent1, parent2, range,  nx);
		childpop.x(i+2,:) = c1;
		childpop.tracking(i+2) = 1 + pop.feas(parent1) + pop.feas(parent2);
		if i+3 <= pop.size
			childpop.x(i+3,:) = c2; 
			childpop.tracking(i+3) = childpop.tracking(i+2);
		end
	end
end

childpop = mutation(childpop, range, nx);


%
% Binary Tournament 
%
function parent = tournament(pop, id1, id2)
if pop.rank(id1) < pop.rank(id2)
	parent = id1;
elseif pop.rank(id2) < pop.rank(id1)
	parent = id2;
elseif rand(1) <= 0.5
	parent = id1;
else
	parent = id2;
end
return


%
% Recombination 
%
function [c1, c2] = recombination(pop, id1, id2, range, nx)
c1 = zeros(nx, 1);
c2 = zeros(nx, 1);

prob_re = pop.algo.prob_re;
eta_re = pop.algo.eta_re;
eta_mc = pop.algo.eta_mc;

do_mc = 0;
if pop.feas(id1) + pop.feas(id2) == 0 && rand(1) <= 1 
%     && length(pop.best.feas) <= pop.algo.generations/2
    do_mc = 1;
end

if rand(1) <= prob_re
    for i = 1:nx
		if rand(1) <= 0.5
            if do_mc == 1
                [y1, y2] = op_MC(pop.x(id1,i), pop.x(id2,i), ...
                        range(i,1), range(i,2), eta_mc);
            else
                [y1, y2] = op_SBX(pop.x(id1,i), pop.x(id2,i), ...
                        range(i,1), range(i,2), eta_re);
            end
		else
			y1 = pop.x(id1,i);
			y2 = pop.x(id2,i);
		end
		c1(i) = y1;
		c2(i) = y2;
    end
else
    c1 = pop.x(id1,:);
    c2 = pop.x(id2,:);
end
return


%
% Mutation
%
function pop = mutation(pop, range, nx)
prob_mu = pop.algo.prob_mu;
eta_mu = pop.algo.eta_mu;

for i = 1:pop.size
    for j = 1:nx
        x_min = range(j,1);
    	x_max = range(j,2);	
		if rand(1) <= prob_mu
			pop.x(i,j) = op_MUTATE(pop.x(i,j), x_min, x_max, eta_mu);
		end
	end
end
return
