function childpop = evolve_realgapcx(pop, user_data)
% EVOLVE_REALGAPCX(pop)
%
childpop = population(pop);

count = floor(pop.size/6 + 0.5);
count2 = floor(pop.size/6);
a1 = randperm(pop.size);
a2 = randperm(pop.size);

range = pop.prob.range;
nx = pop.prob.nx;

% Handle modulo 6 solutions
for i = 1:6:count2*6
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a1(i+3));
	parent3 = tournament(pop, a1(i+4), a1(i+5));
	c = recombination(pop, parent1, parent2, parent3, range, nx);
	childpop.x(i,:) = c(1,:);
	childpop.x(i+1,:) = c(2,:);
	childpop.x(i+2,:) = c(3,:);
	childpop.tracking(i) = 1 + pop.feas(parent1) + pop.feas(parent2);
	childpop.tracking(i+1) = childpop.tracking(i);
	childpop.tracking(i+2) = childpop.tracking(i);
	parent1 = tournament(pop, a2(i)  , a2(i+1));
	parent2 = tournament(pop, a2(i+2), a2(i+3));
	parent3 = tournament(pop, a2(i+4), a2(i+5));
	c = recombination(pop, parent1, parent2, parent3, range,  nx);
	childpop.x(i+3,:) = c(1,:);
	childpop.x(i+4,:) = c(2,:);
	childpop.x(i+5,:) = c(3,:);
	childpop.tracking(i+3) = 1 + pop.feas(parent1) + pop.feas(parent2);
	childpop.tracking(i+4) = childpop.tracking(i+3);
	childpop.tracking(i+5) = childpop.tracking(i+3);
end

% Handle extra solutions 
if count*6 > pop.size
	tmp = randperm(pop.size);
	a1 = [a1 tmp(1:count*6-pop.size)];
	tmp = randperm(pop.size);
	a2 = [a2 tmp(1:count*6-pop.size)];
	
	i = count2*6+1;
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a1(i+3));
	parent3 = tournament(pop, a1(i+4), a1(i+5));
	c = recombination(pop, parent1, parent2, parent3, range, nx);
	childpop.x(i,:) = c(1,:);
	childpop.tracking(i) = 1 + pop.feas(parent1) + pop.feas(parent2);
	if i+1 <= pop.size
		childpop.x(i+1,:) = c(2,:); 
		childpop.tracking(i+1) = childpop.tracking(i);
	end
	if i+2 <= pop.size
		childpop.x(i+2,:) = c(3,:); 
		childpop.tracking(i+2) = childpop.tracking(i);
	end
	if i+3 <= pop.size
		parent1 = tournament(pop, a2(i)  , a2(i+1));
		parent2 = tournament(pop, a2(i+2), a2(i+3));
		parent3 = tournament(pop, a2(i+4), a2(i+5));
		c = recombination(pop, parent1, parent2, parent3, range,  nx);
		childpop.x(i+3,:) = c(1,:);
		childpop.tracking(i+3) = 1 + pop.feas(parent1) + pop.feas(parent2);
		if i+4 <= pop.size
			childpop.x(i+4,:) = c(2,:); 
			childpop.tracking(i+4) = childpop.tracking(i+3);
		end
		if i+5 <= pop.size
			childpop.x(i+5,:) = c(3,:); 
			childpop.tracking(i+5) = childpop.tracking(i+3);
		end
	end
end

childpop = mutation(childpop, range, nx);


%
% Binary Tournament 
%
function parent = tournament(pop, id1, id2)
if pop.rank(id1) < pop.rank(id2)
	parent = id1;
elseif pop.rank(id2) < pop.rank(id1)
	parent = id2;
elseif rand(1) <= 0.5
	parent = id1;
else
	parent = id2;
end
return


%
% Recombination (PCX)
%
function [c] = recombination(pop, id1, id2, id3, range, nx)
c1 = zeros(nx, 1);
c2 = zeros(nx, 1);

x = pop.x([id1,id2,id3],:);
if rand(1) <= pop.algo.prob_re
	c = op_PCX(x, range, pop.algo.eta_pcx, pop.algo.zeta_pcx);
else
    c = x;
end
return


%
% Mutation
%
function pop = mutation(pop, range, nx)
prob_mu = pop.algo.prob_mu;
eta_mu = pop.algo.eta_mu;

for j = 1:nx
	x_min = range(j,1);
	x_max = range(j,2);
	for i = 1:pop.size
		if rand(1) <= prob_mu
			pop.x(i,j) = op_MUTATE(pop.x(i,j), x_min, x_max, eta_mu);
		end
	end
end
return
