function [childpop] = evolve_surrgrad(pop, user_data)

%if isempty(user_data)
%	error('Missing surrogate information (user_data)');
%end

arc = user_data;
eps = 1.e-6;	% get(pop.algo, 'dist_crit'); need new parameter

childpop = evolve_realga(pop, []);

if isempty(arc) || arc.surr.valid == 1
	N = pop.size;

	range = pop.prob.range;

	count = 0;
	for i = 1:N
		x1 = pop.x(i,:);
		[x2,f2,eflag,output] = fmincon(@(x) obj_func(x, pop, arc), x1, ...
				[], [], [], [], range(:,1)', range(:,2)', ...
				@(x) constr_func(x, pop, arc), ...
				optimset('Display', 'off', 'LargeScale', 'off', 'MaxIter', 10));
		fprintf('\t\t%f --> %g (%d) Fn: %d\n', pop.f(i,:), f2, eflag, output.funcCount);
		if eflag >= 0
			flag = 0;
			for j = 1:count
				if norm(x2 - childpop.x(N-j+1,:), 2) < eps
					flag = 1;
					break
				end
			end
			if flag == 0
				childpop.x(N-count,:) = x2;
				count = count+1;
			end
		end
	end
	fprintf('\tMinimized Count = %d\n', count);
end
return


%
% Objective routine for minimizing f on the surrogate subject to constraints
% Penalty formulation
%
function obj = obj_func(x, pop, arc)
[f, g] = obj_constr_eval(pop.prob.func, x, arc);
obj = f;
return


function [c, ceq] = constr_func(x, pop, arc)
[f, g] = obj_constr_eval(pop.prob.func, x, arc);
c = -g;
ceq = [];
return

% Penalty for bounds violation
%for i = 1:length(x)
%	if x(i) < pop.prob.range(i,1)
%		penalty = penalty + 1.e20*abs(pop.prob.range(i,1) - x(i));
%		flag = 1;
%	end
%	if x(i) > pop.prob.range(i,2)
%		penalty = penalty + 1.e20*abs(x(i) - pop.prob.range(i,2));
%		flag = 1;
%	end
%end
