function save(pop, gen, type)
% SAVE(pop,gen) saves population in the file
%
if strcmp(type, 'all')
	save_all(pop, gen);
elseif strcmp(type, 'best')
	save_best(pop, gen);
elseif strcmp(type, 'initial');
	save_initial(pop, gen);
elseif strcmp(type, 'final')
	save_final(pop, gen);
else
	error('No such type');
end
return


% Save All population
function save_all(pop, gen)
filename = strcat(pop.algo.prefix, '-all.dat');
if gen == 1
	fid = fopen(filename, 'w');
else
	fid = fopen(filename, 'a');
end
save_pop(pop, gen, fid);
fclose(fid);
return


% Save initial population
function save_initial(pop, gen)
filename = strcat(pop.algo.prefix, '-initial.dat');
fid = fopen(filename, 'w');
save_pop(pop, gen, fid);
fclose(fid);
return


% Save final population
function save_final(pop, gen)
filename = strcat(pop.algo.prefix, '-final.dat');
fid = fopen(filename, 'w');
save_pop(pop, gen, fid);
fclose(fid);
return


% Save best of the population
function save_best(pop, gen)

filename = strcat(pop.algo.prefix, '-best.dat');
fid = fopen(filename, 'w');

for i = 1:gen
	fprintf(fid, '%4d %5d %3d %2d ', i, pop.best.fn_evals(i), ...
					pop.best.f1size(i), pop.best.feas(i));
	fprintf(fid, ' %g', pop.best.x(i,:));
	fprintf(fid, ' %g', pop.best.f(i,:));
	fprintf(fid, ' %g', pop.best.g(i,:));
	fprintf(fid, '\n');
end
fclose(fid);

global tracking;
if tracking == 1, save_tracking(pop, gen); end
return


% save tracking information of the population
function save_tracking(pop, gen)
filename = strcat(pop.algo.prefix, '-tracking.dat');
fid = fopen(filename, 'w');

for i = 1:gen
	fprintf(fid, '%4d', i);
	fprintf(fid, ' %4d', pop.track_best(i,:));
	fprintf(fid, '\n');
end
fclose(fid);
return


% Save population
function save_pop(pop, gen, fid)
for i = 1:pop.size
	fprintf(fid, '%4d %3d %2d', gen, i, pop.feas(i));
	fprintf(fid, ' %g', pop.x(i,:));
	fprintf(fid, ' %g', pop.f(i,:));
	fprintf(fid, ' %g', pop.g(i,:));
	fprintf(fid, '\n');
end
return
