function [pop_orig, pop] = sort_conga(pop_orig, pop)
% SORT_CONGA(pop_orig, pop)
%		[pop_orig,pop] = sort_conga(pop_orig,pop) sorting for constraint handling
% 
% pop_orig - original population
% pop - modified problem population

N = pop.algo.pop_size;

feasible = [];
infeasible = [];

for i = 1:N
	if pop_orig.feas(i) == 1
		feasible = [feasible i];
	else
		infeasible = [infeasible i];
	end
end

ranks1 = [];
if ~isempty(feasible)
	front_data = non_dominated_sort(pop, feasible);
	pop.f1size = length(front_data.F(1).f);
	pop.F = front_data.F;

	for front = 1 : length(front_data.F)
		[r1, d1] = crowding_distance_sort(pop, front_data, front);
		pop.crowd(front_data.F(front).f(:)) = d1;
		ranks1 = [ranks1 r1];
	end
end

ranks2 = [];
if ~isempty(infeasible)
	front_data = non_dominated_sort(pop, infeasible);
	for front = 1 : length(front_data.F)
		[r1, d1] = crowding_distance_sort(pop, front_data, front);
		pop.crowd(front_data.F(front).f(:)) = d1;
		ranks2 = [ranks2 r1];
	end
end

infeasible_ratio = pop.algo.infeasible_ratio;

n_infeasible = floor(pop.size/2 * infeasible_ratio);
n_feasible = pop.size/2 - n_infeasible;

ranks = [];
rest = [];

if ~isempty(ranks2)
    if length(ranks2) > n_infeasible
    	ranks = [ranks ranks2(1:n_infeasible)];
    	rest = [rest ranks2(n_infeasible+1:end)];
    else
    	ranks = [ranks ranks2];
    end
end

if ~isempty(ranks1)
	ranks = [ranks ranks1];
end

ranks = [ranks rest];
pop.rank = ranks';

pop_orig.f1size = pop.f1size;
pop_orig.F = pop.F;
pop_orig.rank = ranks';

% Append the best solution
global counter;
if isempty(pop_orig.best)
	n = 1;
else
	n = length(pop_orig.best.feas) + 1;
end

% find the id of rank1 solution (as an index in pop.rank)
id = 1;
if pop_orig.feas(pop.rank(id)) ~= 1
	id2 = find(pop_orig.feas(pop_orig.rank) == 1, 1);
	if ~isempty(id2);
		id = id2;
	end
end
pop_orig.rank1 = id;
pop.rank1 = id;

id = pop_orig.rank(id);
pop_orig.best.x(n,:) = pop_orig.x(id,:);
pop_orig.best.f(n,:) = pop_orig.f(id,:);
pop_orig.best.g(n,:) = pop_orig.g(id,:);
pop_orig.best.feas(n) = pop_orig.feas(id);
pop_orig.best.f1size(n) = pop.f1size;
pop_orig.best.fn_evals(n) = counter.fn_evals;

pop.best.x(n,:) = pop.x(id,:);
pop.best.f(n,:) = pop.f(id,:);
pop.best.g(n,:)=  pop.g(id,:);
pop.best.feas(n) = pop_orig.feas(id);
pop.best.f1size(n) = pop.f1size;
pop.best.fn_evals(n) = counter.fn_evals;

return
