%% Latest Version of Cooperative Coevolutionary Algorithm with Adaptive
%% Partitioning based on Variable Correlation

function pop =ccea_avp(config)
    
%     evolve_strat_choice = 1;
    close all;
    prob = problem(config); 
    global counter;
    
    counter = struct('fn_evals', 0);

    % Fixed variables
    algo = paramset(mfilename, config); 
    algo = add_param_fixed(algo, 'evolve_method', 'realga');

    % Parameters for the Sub Population Evolution
    algo = add_param_var(algo, 'pop_size', [], 'INT', [1,inf]);
    algo = add_param_var(algo, 'generations_K', [], 'INT', [1,inf]);
    algo = add_param_var(algo, 'prob_re', [], 'REAL', [0,1]);
    algo = add_param_var(algo, 'eta_re', [], 'REAL', [1,30]);
    algo = add_param_var(algo, 'prob_mu', [], 'REAL', [0,1]);
    algo = add_param_var(algo, 'eta_mu', [], 'REAL', [1,50]);
    algo = add_param_var(algo, 'max_fn_evals', 0, 'INT');
    algo = add_param_var(algo, 'fn_evals', 0, 'INT');
    algo = add_param_var(algo, 'batch_mode', 0, 'BOOL');
    algo = add_param_var(algo, 'seed', [], 'INT', [0,2^32-1]);
    algo = add_param_var(algo, 'no_subsystems', [], 'INT', [0,100]);
    algo = add_param_var(algo, 'no_sub_iterations', [], 'INT', [1, Inf]);
    algo = add_param_var(algo, 'threshold' ,[],'REAL', [0,1]);
    algo = add_param_var(algo, 'partition_strategy',[],'INT',[1 5]);
    algo = add_param_var(algo, 'evolve_strat_choice',[],'INT',[1 5]);    
    algo = load_params(algo);

    %   Initializing the population and evaluating it
    
    pop = population(prob,algo);
    pop = init(pop);
    pop = eval(pop);
    pop = sort(pop);
%     
%     arc = pop.x(1:pop.size/2,:);
%     arc = unique(arc,'rows');
%     
    arc = [pop.x pop.f];
    arc = unique(arc,'rows');
%     arc = sortrows(arc,prob.nx+1);
%     arc = arc(1:floor(size(arc,1)/2));
    
%     arc = pop.x(pop.feas==1,:);
%     b = floor(sqrt(size(arc,1)));
%     ncc = ncie(arc(1:b^2,:),b);
%     entropy(1) = ncie_entropy(ncc,1:prob.nx);
    
    % Running EA with the entire set of variables for K Generations
    for i = 2:algo.generations_K
        childpop = evolve(pop,[]);
        childpop = eval(childpop);
        pop = pop + childpop;
        pop = sort(pop);
        pop = reduce(pop);
        disp(pop, i);
%         arc = [arc; pop.x(1:pop.size/2,:)];
%         arc = unique(arc,'rows');
        arc = [arc; pop.x pop.f];
        arc = unique(arc,'rows');
%         arc = sortrows(arc,prob.nx+1);
%         arc = arc(1:floor(size(arc,1)/2));
    end

    % Identifying the Best in the Population
    best.x=pop.x(1,:);
    best.f=pop.f(1);
    best.feas = pop.feas(1);
    
    fp1 = fopen(sprintf('%s-%s-fxn_evolution.txt',prob.func,mfilename),'w');
    fp2=fopen(sprintf('%s-%s-no_sets.txt',prob.func,mfilename),'w');
    write_to_file([counter.fn_evals pop.f(1) pop.feas(1)],fp1);
    
    % Going for Iterations
    Gen = 1;
    while (true)
        fprintf('CCEA evolve; Generation: %d\n', Gen);
        fprintf('Evaluations: %d\n', counter.fn_evals);

%         arc = unique(arc,'rows');
        temp_arc = sortrows(arc,prob.nx+1);
        temp_arc = temp_arc(1:floor(size(arc,1)/2),1:end-1);
        
%         keyboard
        corr_mat = corr(temp_arc);
%         corr_mat=corr(pop.x(1:pop.size/2,:));
%           corr_mat = corrcoef(pop.x(1:pop.size/2,:));
%         [rho,corr_mat] = corr(pop.x())
        % Find How Many Partions and Variables in Each Partition
        if algo.no_subsystems == 1
            no_sets = 1;
            n_sets_corr = 1;
            partitions.ids = [1:size(corr_mat,2)];
            partitions.nvars = size(corr_mat,2);
        else
            [no_sets,n_sets_corr,partitions]=partition_var(corr_mat,algo,pop);
%             keyboard
%             [no_sets,n_sets_corr,partitions]=partition_pval(corr_mat,algo,pop);            
        end
        if algo.evolve_strat_choice == 1
            for i=1:no_sets
                % Evolving subpopulations
                idx = partitions(i).ids;
                [best,pop] = evolve_subpop('best',idx,pop, prob, algo);
            end
        elseif algo.evolve_strat_choice == 2
            if n_sets_corr == no_sets
                for i = 1:n_sets_corr
                    idx = partitions(i).ids;
                    [best,pop] = evolve_subpop2('best', idx, pop, prob, algo, corr_mat);
                end
            elseif n_sets_corr >=1 && n_sets_corr < no_sets
                for i = 1:n_sets_corr
                    idx = partitions(i).ids;
                    [best,pop] = evolve_subpop2('best', idx, pop, prob, algo, corr_mat);
                end
                for i = n_sets_corr+1:no_sets
                    idx = partitions(i).ids;
                    [best,pop] = evolve_subpop('best', idx, pop, prob, algo);
                end
            else
                for i=1:no_sets
                    idx = partitions(i).ids;
                    [best,pop] = evolve_subpop('best',idx,pop, prob, algo);
                end
            end
        end
            
        pop = eval(pop);
        pop = sort(pop);
%         arc = [arc; pop.x(1:pop.size/2,:)];
%         arc = unique(arc,'rows');
        
        arc = [arc; pop.x pop.f];
        arc = unique(arc,'rows');
%         arc = sortrows(arc,prob.nx+1);
%         arc = arc(1:floor(size(arc,1)/2));        
        
        
        write_to_file([counter.fn_evals pop.f(1) pop.feas(1)], fp1); 
        write_to_file([no_sets n_sets_corr [partitions.nvars]],fp2);
        if(counter.fn_evals>=algo.max_fn_evals)
            break;
        end
        Gen = Gen+1;
    end
    fclose(fp2);
    
return
