%% This function Partitions Variables into Subpopulations based on
%% Correlation
function [no_sets,n_sets_corr,partitions]=partition_pval(corr_mat,algo,pop)

strategy = algo.partition_strategy;
% corr_mat = abs(corr_mat);

[rho, pval] = corr(pop.x);


if strategy == 1
    [no_sets,partitions] = partition_fixed(corr_mat,algo);
    n_sets_corr = 0;
elseif strategy == 2
    [no_sets,partitions] = partition_adaptive(corr_mat,algo);
elseif strategy == 3
    [no_sets,partitions] = partition_adaptive2(corr_mat,algo);
elseif strategy == 4
    [no_sets,partitions] = partition_adaptive3(corr_mat,algo);   
elseif strategy == 5
    [no_sets,n_sets_corr,partitions] = partition_adaptive4(corr_mat,algo);  
elseif strategy == 6
    [no_sets,n_sets_corr,partitions] = partition_adaptive5(corr_mat,pval, algo);     
end

return


function[no_sets,partitions] = partition_fixed(corr_mat,algo)

no_subsystems = algo.no_subsystems;
if no_subsystems == 1
    partitions.ids = 1:size(corr_mat,2);
    partitions.nvars = size(corr_mat,2);
    no_sets = 1;
    return
end

for i = 1:algo.no_subsystems
    partitions(i).ids = [];
    partitions(i).nvars = [];
end

all_ids = 1:size(corr_mat,2);

if rem(length(all_ids),no_subsystems)==0
    n_each = length(all_ids)/no_subsystems;
    for i = 1:no_subsystems
        partitions(i).ids = all_ids((i-1)*n_each+1:i*n_each);
        partitions(i).nvars = n_each;
    end
else
    n_each = floor(length(all_ids)/no_subsystems);
    for i = 1:no_subsystems-1
        partitions(i).ids = all_ids((i-1)*n_each+1:i*n_each);
        partitions(i).nvars = n_each;
    end
    rest_elements = setdiff(all_ids,[partitions(1:no_subsystems-1).ids]);
    partitions(no_subsystems).ids = rest_elements;
    partitions(no_subsystems).nvars = length(rest_elements);
end
    no_sets = algo.no_subsystems; 
return


function [no_sets,n_sets_corr,partitions] = partition_adaptive(corr_mat,algo)
no_subsystems = algo.no_subsystems;
 

if no_subsystems == 1
    partitions.ids = 1:size(corr_mat,2);
    partitions.nvars = size(corr_mat,2);
    no_sets = 1;
    return
end

temp11 = sum(corr_mat>= algo.threshold,2);
[temp2, temp13] = sort(temp11,'descend');


temp3 = find(temp2 == 1);
temp4 = find(temp2 ~= 1);

all_corr = {};

for j=1:length(temp3)+length(temp4)
    all_corr{j}=find((corr_mat(temp13(j),:))>=algo.threshold);
end

if sum(length(temp3))+sum(length(temp4)) <= no_subsystems
    partition_type = 1;
else
    if length(temp4) < no_subsystems
        partition_type = 2;
    elseif length(temp4) >= no_subsystems
        partition_type = 3;
    end
end

switch(partition_type)  
    case 1
        set_no = 0;
        used = [];
        for i = 1:length(temp4)+length(temp3)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
            end
        end
    case 2
        for i = 1:no_subsystems
            partitions(i).ids = [];
            partitions(i).nvars = 0;
        end
        used = [];
        set_no = 0;
        for i = 1:length(temp4)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
            end
        end
        
        rest_partitions = no_subsystems - set_no;
        rem_ids = setdiff(1:size(corr_mat,1), used);
        
        rem_ids = unique(rem_ids);
        
        if rem(length(rem_ids),rest_partitions)==0
            n_each = length(rem_ids)/rest_partitions;
            for i = 1:rest_partitions
                new_elements = rem_ids((i-1)*n_each+1:i*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        else
            n_each = floor(length(rem_ids)/rest_partitions);
            balance_vars = rem(length(rem_ids), rest_partitions);  
            for i = 1:balance_vars
                new_elements = rem_ids((i-1)*(n_each+1)+1:i*(n_each+1));
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
            done = balance_vars*(n_each+1);
            for i = balance_vars+1:rest_partitions
                new_elements = rem_ids(done+(i-balance_vars-1)*n_each+1:done+(i-balance_vars)*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        end            
        
    case 3
        for i = 1:no_subsystems
            partitions(i).ids = [];
            partitions(i).nvars = 0;
        end        
        used = [];
        set_no = 0;
        flag = 0;
%         break_point = length(temp4);
        for i = 1:length(temp4)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
                if set_no == no_subsystems-1
                   flag = 1;
                end
            end
            if flag == 1 
                break_point = i;           
                break;
            end
        end
        
        rest_partitions = no_subsystems - set_no;
        rem_ids = setdiff(1:size(corr_mat,1), used);
        
        rem_ids = unique(rem_ids);
        
        if rem(length(rem_ids),rest_partitions)==0
            n_each = length(rem_ids)/rest_partitions;
            for i = 1:rest_partitions
                new_elements = rem_ids((i-1)*n_each+1:i*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        else
            n_each = floor(length(rem_ids)/rest_partitions);
            balance_vars = rem(length(rem_ids), rest_partitions);  
            for i = 1:balance_vars
                new_elements = rem_ids((i-1)*(n_each+1)+1:i*(n_each+1));
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
            done = balance_vars*(n_each+1);
            for i = balance_vars+1:rest_partitions
               new_elements = rem_ids(done+(i-balance_vars-1)*n_each+1:done+(i-balance_vars)*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        end      

end

fprintf('Correlation based: %d; Rest : %d\n', no_subsystems-rest_partitions, rest_partitions);
fprintf('Elements in each : %s; Total = %d\n', num2str([partitions.nvars]), length([partitions.ids]));
no_sets = length(partitions);
n_sets_corr = no_subsystems-rest_partitions;

if ~isempty(setdiff(1:size(corr_mat,2), [partitions.ids]));
    error('Error in partitioning \n');
end

return

function [no_sets,partitions] = partition_adaptive2(corr_mat,algo)
no_subsystems = algo.no_subsystems;

if no_subsystems == 1
    partitions.ids = 1:size(corr_mat,2);
    partitions.nvars = size(corr_mat,2);
    no_sets = 1;
    return
end

temp11 = sum(corr_mat>= algo.threshold,2);
[temp2, temp13] = sort(temp11,'descend');


temp3 = find(temp2 == 1);
temp4 = find(temp2 ~= 1);

all_corr = {};

for j=1:length(temp3)+length(temp4)
    all_corr{j}=find((corr_mat(temp13(j),:))>=algo.threshold);
end

if sum(length(temp3))+sum(length(temp4)) <= no_subsystems
    partition_type = 1;
else
    if length(temp4) < no_subsystems
        partition_type = 2;
    elseif length(temp4) >= no_subsystems
        partition_type = 3;
    end
end

switch(partition_type)  
    case 1
        set_no = 0;
        used = [];
        for i = 1:length(temp4)+length(temp3)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
            end
        end
    case 2
        used = [];
        set_no = 0;
        for i = 1:length(temp4)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
            end
        end
      
        rest_elements = setdiff(1:size(corr_mat,2),used);
        if ~isempty(rest_elements)
            set_no = set_no+1;
            partitions(set_no).ids = rest_elements;
            partitions(set_no).nvars = length(rest_elements);
        end
        
    case 3
        for i = 1:no_subsystems
            partitions(i).ids = [];
            partitions(i).nvars = 0;
        end        
        used = [];
        set_no = 0;
        flag = 0;
        for i = 1:length(temp4)
            if ~isempty(setdiff(all_corr{i}, used))
                set_no = set_no + 1;
                new_elements = setdiff(all_corr{i},used);
                partitions(set_no).ids = new_elements;
                partitions(set_no).nvars = length(new_elements);
                used = [used new_elements];
                if set_no == no_subsystems-1
                   flag = 1;
                end
            end
            if flag == 1 
                break;
            end
        end
        
        rest_elements = setdiff(1:size(corr_mat,2),used);
        if ~isempty(rest_elements)
            set_no = set_no+1;
            partitions(set_no).ids = rest_elements;
            partitions(set_no).nvars = length(rest_elements);
        end   

end

no_sets = length(partitions);

return

function [no_sets,partitions] = partition_adaptive3(corr_mat,algo)
no_subsystems = algo.no_subsystems;

if no_subsystems == 1
    partitions.ids = 1:size(corr_mat,2);
    partitions.nvars = size(corr_mat,2);
    no_sets = 1;
    return
end

ids = 1:size(corr_mat,2);

if no_subsystems == 1
    partitions.ids = 1:size(corr_mat,2);
    partitions.nvars = size(corr_mat,2);
    no_sets = 1;
    return
end

temp11 = sum(corr_mat>= algo.threshold,2);

non_corr_ids = find(temp11)==1;
[temp2, temp13] = sort(temp11,'descend');


temp3 = find(temp2 == 1);
temp4 = find(temp2 ~= 1);
% 
% all_corr = {};
% 
% for j=1:length(temp3)+length(temp4)
%     all_corr{j}=find((corr_mat(temp13(j),:))>=algo.threshold);
% end

set_no = 1;
% j = 1;
used_ids = [];
remain_ids = corr_mat(temp13(temp4),:);
while ~isempty(remain_ids)
    corr_ids = find(remain_ids(1,:) == 1);
    temp = corr_ids;
    for i = 1:length(corr_ids)
        temp = [temp find(corr_mat(corr_ids(i),:)>=algo.threshold)];
    end
    temp = unique(temp);
    
    set_no = set_no+1;
    partitions(set_no).ids = temp;
    partitions(set_no).nvars = length(temp);
    remain_ids(temp,:) = [];
    used_ids = [used_ids temp];

    if ~isempty(setdiff(1:size(corr_mat,2), used_ids)) && set_no == no_subsystems - 1
        rest_ids = setdiff(1:size(corr_mat,2), used);
        set_no = set_no+1;
        partitions(set_no).ids = rest_ids;
        partitions(set_no).nvars = length(rest_ids);
        used_ids = [used_ids rest_ids];
        break
    end
end

        rest_partitions = no_subsystems - set_no;
        rem_ids = setdiff(1:size(corr_mat,1), used);
        if rest_partitions > 0
            if length(rem_ids)>rest_partitions        
                if rem(length(rem_ids),rest_partitions)==0
                    n_each = length(rem_ids)/rest_partitions;
                    for i = 1:rest_partitions
                        new_elements = rem_ids((i-1)*n_each+1:i*n_each);
                        partitions(set_no+i).ids = new_elements;
                        partitions(set_no+i).nvars = length(new_elements);
                    end
                else
                    n_each = floor(length(rem_ids)/rest_partitions);
                    balance_vars = rem(length(rem_ids), rest_partitions);  
                    for i = 1:balance_vars
                        new_elements = rem_ids((i-1)*(n_each+1)+1:i*(n_each+1));
                        partitions(set_no+i).ids = new_elements;
                        partitions(set_no+i).nvars = length(new_elements);
                    end
                    done = balance_vars*(n_each+1);
                    for i = balance_vars+1:rest_partitions
                       new_elements = rem_ids(done+(i-balance_vars-1)*n_each+1:done+(i-balance_vars)*n_each);
                        partitions(set_no+i).ids = new_elements;
                        partitions(set_no+i).nvars = length(new_elements);
                    end
                end
            end
        else
            for i = 1: length(rem_ids)
                partitions(set_no+i).ids = rem_ids(i);
                partitions(set_no+i).nvars = 1;
            end
        end

return


function [no_sets,n_sets_corr,partitions] = partition_adaptive4(corr_mat,algo)

no_subsystems = algo.no_subsystems;

exc_th = corr_mat >= algo.threshold;
temp2 = sum(corr_mat>= algo.threshold,2);

temp3 = find(temp2 == 1);
temp4 = find(temp2 ~= 1);

set_no = 0;

partitions.ids = [];
partitions.nvars = [];

for i = 1:size(corr_mat,2)
    if ~ismember(i,temp3)
        if set_no == 0
            set_no = 1;
            partitions(set_no).ids = find(exc_th(i,:)==1);
        else
            temp = find(exc_th(i,:)==1);
            flag = 0;
            for j = 1:length(partitions)
                if ~isempty(intersect(temp,partitions(j).ids))
                    partitions(j).ids = unique([partitions(j).ids temp]);
                    flag = 1;
                    break
                end
            end
            if flag == 0
                    set_no = set_no+1;
                    partitions(set_no).ids = temp;
            end
        end
    end
end

non_empty_ids = [];
for i = 1:length(partitions)
    if ~isempty(partitions(i).ids)
        non_empty_ids = [non_empty_ids i];
    end
end
partitions = partitions(non_empty_ids);


if length(partitions) > 1
    common_ids = {};
    nsets = 1;
    used = [];
    for i = 1:length(partitions)
        if ~ismember(i,used)
            common_ids{nsets} = i;
            used = [used i];
            for j = i+1:length(partitions)
                if ~ismember(j,used)
                    if ~isempty(intersect([partitions(common_ids{nsets}).ids], partitions(j).ids))
                        common_ids{nsets} = [common_ids{nsets} j];
                        used = [used j];
%                         keyboard
                    end
%                     keyboard
                end
            end
            nsets = nsets+1;
        end
    end

    for i = 1:length(common_ids)
        temp_sets(i).ids = unique([partitions(common_ids{i}).ids]);
    end
    
%     keyboard
%     keyboard
    partitions = temp_sets;

    for i = 1:length(partitions)
        partitions(i).nvars = length(partitions(i).ids);
    end
    set_no = length(partitions);
%     keyboard
end


if length(partitions)==1
    if ~isempty(partitions.ids)
        partitions.nvars = length(partitions.ids);
        set_no = 1;
    else
        set_no = 0;        
    end
end



if length(partitions) >= no_subsystems
    [tmp,I] = sort([partitions.nvars]);
    partitions = partitions(I);
    extra_partition_ids = [[partitions(no_subsystems:end).ids] temp3'];
    
    for i = 1:no_subsystems - 1
        final_partitions(i).ids = partitions(i).ids;
    end
    final_partitions(no_subsystems).ids = extra_partition_ids;
    final_partitions(no_subsystems).nvars = length(extra_partition_ids);
    
    for i = 1:no_subsystems-1
        final_partitions(i).nvars = length(final_partitions(i).ids);
    end
    partitions = final_partitions;
    rest_partitions = 1;
else
    rest_partitions = no_subsystems - set_no;
    rem_ids = temp3';
    if length(rem_ids) < rest_partitions
        for i = 1:length(rem_ids)
            partitions(set_no+i).ids = temp3(i);
            partitions(set_no+i).nvars = 1;
        end
    else
        if rem(length(rem_ids),rest_partitions)==0
            n_each = length(rem_ids)/rest_partitions;
            for i = 1:rest_partitions
                new_elements = rem_ids((i-1)*n_each+1:i*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        else
            n_each = floor(length(rem_ids)/rest_partitions);
            balance_vars = rem(length(rem_ids), rest_partitions);  
            for i = 1:balance_vars
                new_elements = rem_ids((i-1)*(n_each+1)+1:i*(n_each+1));
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
            done = balance_vars*(n_each+1);
            for i = balance_vars+1:rest_partitions
               new_elements = rem_ids(done+(i-balance_vars-1)*n_each+1:done+(i-balance_vars)*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        end
    end
end

if ~isempty(setdiff(1:size(corr_mat,2), [partitions.ids]))
%     error('Error in partitioning \n');
    unresolved_ids = setdiff(1:size(corr_mat,2), [partitions.ids]);
    partitions(end).ids = unique([partitions(end).ids unresolved_ids]);
    partitions(end).nvars = length(partitions(end).ids);
end

if ~isempty(rest_partitions)
    fprintf('Correlation based: %d; Rest : %d\n', no_subsystems-rest_partitions, rest_partitions);
    fprintf('Elements in each : %s; Total = %d\n', num2str([partitions.nvars]), length([partitions.ids]));
else
    fprintf('No. of correlation based partitions > no_subsystems\n')
    fprintf('Elements in each : %s; Total = %d\n', num2str([partitions.nvars]), length([partitions.ids]));    
end
    no_sets = length(partitions);
    n_sets_corr = no_subsystems - rest_partitions;
%     keyboard


return


function [no_sets,n_sets_corr,partitions] = partition_adaptive5(corr_mat,pval,algo)

no_subsystems = algo.no_subsystems;

exc_th = pval <= 0.02;
temp2 = sum(pval <= 0.02,2);

temp3 = find(temp2 == 1);
temp4 = find(temp2 ~= 1);

set_no = 0;

partitions.ids = [];
partitions.nvars = [];

for i = 1:size(corr_mat,2)
    if ~ismember(i,temp3)
        if set_no == 0
            set_no = 1;
            partitions(set_no).ids = find(exc_th(i,:)==1);
        else
            temp = find(exc_th(i,:)==1);
            flag = 0;
            for j = 1:length(partitions)
                if ~isempty(intersect(temp,partitions(j).ids))
                    partitions(j).ids = unique([partitions(j).ids temp]);
                    flag = 1;
                    break
                end
            end
            if flag == 0
                    set_no = set_no+1;
                    partitions(set_no).ids = temp;
            end
        end
    end
end

non_empty_ids = [];
for i = 1:length(partitions)
    if ~isempty(partitions(i).ids)
        non_empty_ids = [non_empty_ids i];
    end
end
partitions = partitions(non_empty_ids);


if length(partitions) > 1
    common_ids = {};
    nsets = 1;
    used = [];
    for i = 1:length(partitions)
        if ~ismember(i,used)
            common_ids{nsets} = i;
            used = [used i];
            for j = i+1:length(partitions)
                if ~ismember(j,used)
                    if ~isempty(intersect([partitions(common_ids{nsets}).ids], partitions(j).ids))
                        common_ids{nsets} = [common_ids{nsets} j];
                        used = [used j];
%                         keyboard
                    end
%                     keyboard
                end
            end
            nsets = nsets+1;
        end
    end

    for i = 1:length(common_ids)
        temp_sets(i).ids = unique([partitions(common_ids{i}).ids]);
    end
    
%     keyboard
%     keyboard
    partitions = temp_sets;

    for i = 1:length(partitions)
        partitions(i).nvars = length(partitions(i).ids);
    end
    set_no = length(partitions);
%     keyboard
end


if length(partitions)==1
    if ~isempty(partitions.ids)
        partitions.nvars = length(partitions.ids);
        set_no = 1;
    else
        set_no = 0;        
    end
end



if length(partitions) >= no_subsystems
    [tmp,I] = sort([partitions.nvars]);
    partitions = partitions(I);
    extra_partition_ids = [[partitions(no_subsystems:end).ids] temp3'];
    
    for i = 1:no_subsystems - 1
        final_partitions(i).ids = partitions(i).ids;
    end
    final_partitions(no_subsystems).ids = extra_partition_ids;
    final_partitions(no_subsystems).nvars = length(extra_partition_ids);
    
    for i = 1:no_subsystems-1
        final_partitions(i).nvars = length(final_partitions(i).ids);
    end
    partitions = final_partitions;
    rest_partitions = 1;
else
    rest_partitions = no_subsystems - set_no;
    rem_ids = temp3';
    if length(rem_ids) < rest_partitions
        for i = 1:length(rem_ids)
            partitions(set_no+i).ids = temp3(i);
            partitions(set_no+i).nvars = 1;
        end
    else
        if rem(length(rem_ids),rest_partitions)==0
            n_each = length(rem_ids)/rest_partitions;
            for i = 1:rest_partitions
                new_elements = rem_ids((i-1)*n_each+1:i*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        else
            n_each = floor(length(rem_ids)/rest_partitions);
            balance_vars = rem(length(rem_ids), rest_partitions);  
            for i = 1:balance_vars
                new_elements = rem_ids((i-1)*(n_each+1)+1:i*(n_each+1));
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
            done = balance_vars*(n_each+1);
            for i = balance_vars+1:rest_partitions
               new_elements = rem_ids(done+(i-balance_vars-1)*n_each+1:done+(i-balance_vars)*n_each);
                partitions(set_no+i).ids = new_elements;
                partitions(set_no+i).nvars = length(new_elements);
            end
        end
    end
end

if ~isempty(setdiff(1:size(corr_mat,2), [partitions.ids]))
%     error('Error in partitioning \n');
    unresolved_ids = setdiff(1:size(corr_mat,2), [partitions.ids]);
    partitions(end).ids = unique([partitions(end).ids unresolved_ids]);
    partitions(end).nvars = length(partitions(end).ids);
end

if ~isempty(rest_partitions)
    fprintf('Correlation based: %d; Rest : %d\n', no_subsystems-rest_partitions, rest_partitions);
    fprintf('Elements in each : %s; Total = %d\n', num2str([partitions.nvars]), length([partitions.ids]));
else
    fprintf('No. of correlation based partitions > no_subsystems\n')
    fprintf('Elements in each : %s; Total = %d\n', num2str([partitions.nvars]), length([partitions.ids]));    
end
    no_sets = length(partitions);
    n_sets_corr = no_subsystems - rest_partitions;
%     keyboard


return



