#include <malloc.h>

#include "mex.h"

void 
indsort(int N, int *feasible, int nf, double *f, double *ind_N, 
				double *ind_S, double *ind_S_count)
{
int i, j, k;
int id1, id2;
double *f1, *f2;
double *i1, *i2;
int dom_less, dom_more;

	/* Initalize all output variables */
	for(i=0; i<N; i++) {
		ind_N[i] = 0;
		ind_S_count[i] = 0;
		for(j=0; j<N; j++) {
			ind_S[i*N+j] = -1;
		}
	}

	/* NDS fill routine */
	for(i=0; i<N; i++) {
		id1 = feasible[i];
		f1 = &f[i*nf];
		for(j=0; j<N; j++) {
			if(i == j) continue;
			id2 = feasible[j];
			f2 = &f[j*nf];
			dom_less = 0;
			dom_more = 0;
			for(k=0; k<nf; k++) {
				if(f1[k] <= f2[k]) dom_less++;
				if(f1[k] >= f2[k]) dom_more++;
			}
			if(dom_less == nf && dom_more < nf) {
				i1 = &ind_S[i*N];
				i1[(int)ind_S_count[i]] = (double)id2;
				ind_S_count[i]++;
			} else if(dom_more == nf && dom_less < nf) {
				ind_N[i]++;
			}
		}
	}
}



void 
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
int i;
int mrows, ncols;
int N, nf;
int *feasible;
double *f, *ptr;
double *ind_N, *ind_S, *ind_S_count;

	/* Check if proper number of arguments */
	if(nrhs != 2) {
		mexErrMsgTxt("Two Inputs are required.");
	} else if(nlhs != 3) {
		mexErrMsgTxt("Three outputs are returned.");
	}

	/* First input is the feasible indices */
	mrows = mxGetM(prhs[0]);
	ncols = mxGetN(prhs[0]);
	if(!mxIsDouble(prhs[0]) || !(mrows == 1)) {
		mexErrMsgTxt("First input must be a column array of integers.");
	}
	N = ncols;
	ptr = (double *)mxGetPr(prhs[0]);
	feasible = malloc(N * sizeof(int));
	for(i=0; i<N; i++) feasible[i] = (int)ptr[i];

	/* Second input is the objective array */
	mrows = mxGetM(prhs[1]);
	ncols = mxGetN(prhs[1]);
	if(!mxIsDouble(prhs[1]) || !(ncols == N)) {
		mexErrMsgTxt("Second input must be an array of doubles.");
	}
	nf = mrows;
	f = (double *)mxGetPr(prhs[1]);

	/* Create output variables */
	plhs[0] = mxCreateDoubleMatrix(N, 1, mxREAL);
	plhs[1] = mxCreateDoubleMatrix(N, N, mxREAL);
	plhs[2] = mxCreateDoubleMatrix(N, 1, mxREAL);

	ind_N = (double *)mxGetPr(plhs[0]);
	ind_S = (double *)mxGetPr(plhs[1]);
	ind_S_count = (double *)mxGetPr(plhs[2]);

	indsort(N, feasible, nf, f, ind_N, ind_S, ind_S_count);
}
