#include <malloc.h>
#include <assert.h>
#include <math.h>

#include "mex.h"

static const void *_data;


int
compare(const void *a, const void *b)
{
int id1, id2;
double f1, f2;

	id1 = *(int *)a;
	id2 = *(int *)b;

	f1 = ((double *)_data)[id1];
	f2 = ((double *)_data)[id2];

	return (f1 < f2 ? -1 : (f1 > f2 ? 1 : 0));
}


void
ncie(double **X, int b, int N, int nd, int **ID, double **NCC)
{
int M, id, i, j, k, p, q;
int *ranks1, *ranks2;
int **rank_grid;
double tmp;

	ranks1 = (int *)malloc(N * sizeof(int));
	ranks2 = (int *)malloc(N * sizeof(int));

	rank_grid = (int **)malloc(b * sizeof(int *));
	for(i=0; i<b; i++) {
		rank_grid[i] = (int *)malloc(b * sizeof(int));
	}

	M = N / b;

	for(i=0; i<nd; i++) {
		for(j=0; j<nd; j++) {
			if(i==j) {
				NCC[i][j] = 1;
			} else {
				NCC[i][j] = 0;
			}
		}
	}

	for(i=0; i<nd; i++) {
		_data = X[i];
		for(j=0; j<N; j++) ID[i][j] = j;
		qsort(ID[i], N, sizeof(int), compare);
	}

	for(i=0; i<nd-1; i++) {
		for(j=i+1; j<nd; j++) {

			for(k=0; k<N; k++) {
				ranks1[k] = 0;
				ranks2[k] = 0;
			}

			for(k=0; k<N; k++) {
				id = (int)(k / b);
				ranks1[ID[i][k]] = id;
				ranks2[ID[j][k]] = id;
			}

			for(p=0; p<b; p++) {
				for(q=0; q<b; q++) {
					rank_grid[p][q] = 0;
				}
			}

			for(k=0; k<N; k++) {
				rank_grid[ranks1[k]][ranks2[k]]++;
			}

			for(p=0; p<b; p++) {
				for(q=0; q<b; q++) {
					if(rank_grid[p][q] != 0) {
						tmp = ((double)rank_grid[p][q] * log((double)rank_grid[p][q]/N));
						NCC[i][j] += tmp / (N*log((double)b));
					}
				}
			}

			NCC[i][j] += 2;
			NCC[j][i] = NCC[i][j];
		}
	}

	for(i=0; i<b; i++) {
		free(rank_grid[i]);
	}
	free(rank_grid);
	free(ranks1);
	free(ranks2);
}


void 
mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
int i, j, k, b;
int mrows, ncols;
int N, nd;
double **X, **NCC;
int **ID;
double *ptr;

	/* Check if proper number of arguments */
	if(nrhs != 2) {
		mexErrMsgTxt("Two Inputs are required.");
	} else if(nlhs != 1) {
		mexErrMsgTxt("One output is returned.");
	}

	/* First input is the X */
	mrows = mxGetM(prhs[0]);
	ncols = mxGetN(prhs[0]);
	if(!mxIsDouble(prhs[0])) {
		mexErrMsgTxt("First input must be an array of doubles.");
	}
	nd = ncols;
	N = mrows;
	ptr = (double *)mxGetPr(prhs[0]);

	/* Second input is scalar b */
	mrows = mxGetM(prhs[1]);
	ncols = mxGetN(prhs[1]);
	if(!mxIsDouble(prhs[1]) || !(ncols == 1 && mrows == 1)) {
		mexErrMsgTxt("Second input must be a scalar.");
	}
	b = (int)*(double *)mxGetPr(prhs[1]);

	/* Create input variable */
	X = (double **)malloc(nd * sizeof(double *));
	for(i=0; i<nd; i++) {
		X[i] = (double *)malloc(N * sizeof(double));
	}
	
	/* Create temporary variables */
	ID = (int **)malloc(nd * sizeof(int *));
	for(i=0; i<nd; i++) {
		ID[i] = (int *)malloc(N * sizeof(int));
	}

	/* Create output variable */
	NCC = (double **)malloc(nd * sizeof(double *));
	for(i=0; i<nd; i++) {
		NCC[i] = (double *)malloc(nd * sizeof(double));
	}

	assert(X != NULL);
	assert(ID != NULL);
	assert(NCC != NULL);

	/* Copy input variable */
	k = 0;
	for(i=0; i<nd; i++) {
		for(j=0; j<N; j++) {
			X[i][j] = ptr[k];
			k++;
		}
	}

			
	/* Create output variables */
	plhs[0] = mxCreateDoubleMatrix(nd, nd, mxREAL);
	ptr = (double *)mxGetPr(plhs[0]);

	ncie(X, b, N, nd, ID, NCC);

	/* Copy output variable */
	k = 0;
	for(i=0; i<nd; i++) {
		for(j=0; j<nd; j++) {
			ptr[k] = NCC[j][i];	
			k++;
		}
	}

	for(i=0; i<nd; i++) {
		free(X[i]);
		free(ID[i]);
		free(NCC[i]);
	}
	free(X);
	free(ID);
	free(NCC);
}
