function [f, g, true_eval] = obj_constr_eval(func, x, user_data)
% OBJ_CONSTR_EVAL(func, x)
%		obj_constr_eval(func, x)  Evaluation of Objectives and Constraints

% Use user_data to evaluate the objective and constraints

global counter;

if nargin == 2
	user_data = [];
end

true_eval = 1;

if ~isempty(user_data)
	arc = user_data;

	[f, g, valid] = predict(arc, x);
	if valid == 1
		true_eval = 0;
	end
end

use_cache = 0;

if true_eval == 1
	counter.fn_evals = counter.fn_evals + 1;
	if use_cache == 1
		try
			[f,g] = cache_eval(func, x);
		catch
			[f,g] = feval(func, x);
			cache_save(func, x, f, g);
		end
	else
		[f,g] = feval(func, x);
	end
end
return



%% Find if we have cached the solution
function [f,g] = cache_eval(func, x)
filename = sprintf('%s-cache.mat', func);
load(filename);

found = 0;
m = size(x,2);
for i = 1:size(cache.x,1)
	if sum(cache.x(i,:) == x) == m
		f = cache.f(i,:);
		g = cache.g(i,:);
		found = 1;
		break
	end
end
if ~found
	error('Not in Cache');
end
return


%% save the cache
function cache_save(func, x, f, g)
filename = sprintf('%s-cache.mat', func);
try
	load(filename);
	cache.x = [cache.x ; x];
	cache.f = [cache.f ; f];
	cache.g = [cache.g ; g];
catch
	cache = [];
	cache.x = x;
	cache.f = f;
	cache.g = g;
end
save(filename, 'cache');
return
