function [c1, c2] = op_SBX_mod(x1, x2, x_min, x_max, eta_re, dir_choice)

if dir_choice == 1
    [c1,c2] = do_SBX_reduce(x1, x2, x_min, x_max, eta_re);
elseif dir_choice == 2
    [c1,c2] = do_SBX_increase(x1, x2, x_min, x_max, eta_re);
end

return
function [c1,c2] = do_SBX_reduce(x1, x2, x_min, x_max, eta_re)
if abs(x1-x2) > 1.e-6
    if x2 < x1, tmp = x1; x1 = x2; x2 = tmp; end
    
    c1 = x1;
    c2 = x2;
    
    while (c1 >= x1)
        r = rand(1);
        beta = 1 + (2 * (x1-x_min) / (x2-x1));
        alpha = 2 - beta^-(eta_re+1);
        if r <= 1/alpha
            betaq = (r*alpha)^(1/(eta_re+1));
        else
            betaq = (1/(2-r*alpha))^(1/(eta_re+1));
        end
        c1 = 0.5 * (x1+x2 - betaq*(x2-x1));
    end
    while (c2 >= x2)
        r = rand(1);
        beta = 1 + (2 * (x_max-x2) / (x2-x1));
        alpha = 2 - beta^-(eta_re+1);
        if r <= 1/alpha
            betaq = (r*alpha)^(1/(eta_re+1));
        else
            betaq = (1/(2-r*alpha))^(1/(eta_re+1));
        end
        c2 = 0.5 * (x1+x2 + betaq*(x2-x1));
    end
    
	if c1 < x_min, c1 = x_min; end
	if c1 > x_max, c1 = x_max; end

	if c2 < x_min, c2 = x_min; end
	if c2 > x_max, c2 = x_max; end

	if rand(1) <= 0.5, tmp = c1; c1 = c2; c2 = tmp; end
else
	c1 = x1;
	c2 = x2;
end
return

function [c1,c2] = do_SBX_increase(x1, x2, x_min, x_max, eta_re)
if abs(x1-x2) > 1.e-6
    if x2 < x1, tmp = x1; x1 = x2; x2 = tmp; end
    
    c1 = x1;
    c2 = x2;
    
    while (c1 <= x1)
        r = rand(1);
        beta = 1 + (2 * (x1-x_min) / (x2-x1));
        alpha = 2 - beta^-(eta_re+1);
        if r <= 1/alpha
            betaq = (r*alpha)^(1/(eta_re+1));
        else
            betaq = (1/(2-r*alpha))^(1/(eta_re+1));
        end
        c1 = 0.5 * (x1+x2 - betaq*(x2-x1));
    end
    while (c2 <= x2)
        r = rand(1);
        beta = 1 + (2 * (x_max-x2) / (x2-x1));
        alpha = 2 - beta^-(eta_re+1);
        if r <= 1/alpha
            betaq = (r*alpha)^(1/(eta_re+1));
        else
            betaq = (1/(2-r*alpha))^(1/(eta_re+1));
        end
        c2 = 0.5 * (x1+x2 + betaq*(x2-x1));
    end
    
	if c1 < x_min, c1 = x_min; end
	if c1 > x_max, c1 = x_max; end

	if c2 < x_min, c2 = x_min; end
	if c2 > x_max, c2 = x_max; end

	if rand(1) <= 0.5, tmp = c1; c1 = c2; c2 = tmp; end
else
	c1 = x1;
	c2 = x2;
end
return